@@ -14,10 +14,10 @@ from gi.repository import GLib, GdkPixbuf, Gio
 
 from hashlib import md5
 
-from lollypop.define import ArtBehaviour, ArtSize
-from lollypop.define import CACHE_PATH
+from lollypop.define import ArtBehaviour, ArtSize, App
+from lollypop.define import CACHE_PATH, ARTISTS_PATH
 from lollypop.logger import Logger
-from lollypop.utils import escape, emit_signal
+from lollypop.utils import emit_signal, escape, get_default_storage_type
 
 
 class ArtistArt:
@@ -26,53 +26,38 @@ class ArtistArt:
          Should be inherited by a BaseArt
     """
 
-    _INFO_PATH = GLib.get_user_data_dir() + "/lollypop/info"
-
     def __init__(self):
         """
-            Init album art
+            Init artist artwork
         """
         try:
-            d = Gio.File.new_for_path(self._INFO_PATH)
-            if not d.query_exists():
-                d.make_directory_with_parents()
-        except:
-            Logger.info("Can't create %s" % self._INFO_PATH)
+            self.__migrate_old_dir()
+        except Exception as e:
+            Logger.error("ArtistArt::__init__(): %s", e)
 
-    def artist_artwork_exists(self, artist):
+    def get_artist_artwork_path(self, artist):
         """
             True if artist artwork exists
             @param artist as str
-            @return (bool, path)
+            @return str/None
         """
-        escaped_artist = escape(artist)
-        # Search in store
-        paths = ["%s/%s.jpg" % (
-                 self._INFO_PATH,
-                 escaped_artist),
-                 "%s/web_%s.jpg" % (
-                 self._INFO_PATH,
-                 escaped_artist)]
-        for path in paths:
-            if GLib.file_test(path, GLib.FileTest.EXISTS):
-                return (True, path)
-        return (False, None)
+        encoded = self.encode_artist_name(artist)
+        filepath = "%s/%s.jpg" % (ARTISTS_PATH, encoded)
+        if GLib.file_test(filepath, GLib.FileTest.EXISTS):
+            return filepath
+        return None
 
-    def add_artist_artwork(self, artist, data, is_web=False):
+    def add_artist_artwork(self, artist, data, storage_type):
         """
             Add artist artwork to store
             @param artist as str
             @param data as bytes
-            @param is_web as bool
+            @param storage_type as StorageType
             @thread safe
         """
         self.uncache_artist_artwork(artist)
-        if is_web:
-            filepath = "%s/web_%s.jpg" % (self._INFO_PATH,
-                                          escape(artist))
-        else:
-            filepath = "%s/%s.jpg" % (self._INFO_PATH,
-                                      escape(artist))
+        encoded = self.encode_artist_name(artist)
+        filepath = "%s/%s.jpg" % (ARTISTS_PATH, encoded)
         if data is None:
             f = Gio.File.new_for_path(filepath)
             fstream = f.replace(None, False,
@@ -113,7 +98,7 @@ class ArtistArt:
         else:
             w = width
             h = height
-        filename = self.get_artist_cache_name(artist)
+        filename = self.encode_artist_name(artist)
         cache_path_jpg = "%s/%s_%s_%s.jpg" % (CACHE_PATH, filename, w, h)
         pixbuf = None
         try:
@@ -126,10 +111,10 @@ class ArtistArt:
                                                  width, height, behaviour)
                 return pixbuf
             else:
-                (exists, path) = self.artist_artwork_exists(artist)
-                if exists:
+                filepath = self.get_artist_artwork_path(artist)
+                if filepath is not None:
                     try:
-                        pixbuf = GdkPixbuf.Pixbuf.new_from_file(path)
+                        pixbuf = GdkPixbuf.Pixbuf.new_from_file(filepath)
                     except:
                         return None
                 else:
@@ -142,7 +127,7 @@ class ArtistArt:
             Logger.error("ArtistArt::get_artist_artwork(): %s" % e)
             return None
 
-    def get_artist_cache_name(self, artist):
+    def encode_artist_name(self, artist):
         """
             Get a uniq string for artist
             @param artist as str
@@ -156,7 +141,7 @@ class ArtistArt:
         """
         try:
             from pathlib import Path
-            search = "%s*.jpg" % self.get_artist_cache_name(artist)
+            search = "%s*.jpg" % self.encode_artist_name(artist)
             for p in Path(CACHE_PATH).glob(search):
                 p.unlink()
         except Exception as e:
@@ -165,3 +150,30 @@ class ArtistArt:
 #######################
 # PRIVATE             #
 #######################
+    def __migrate_old_dir(self):
+        """
+            Migrate old data dir
+        """
+        try:
+            old_path = GLib.get_user_data_dir() + "/lollypop/info"
+            old_src = Gio.File.new_for_path(old_path)
+            if not old_src.query_exists():
+                return
+            storage_type = get_default_storage_type()
+            for (artist_id, artist, *ignore) in App().artists.get(
+                    [], storage_type):
+                for ext in ["jpg", "txt"]:
+                    src_path = "%s/%s.%s" % (old_path, escape(artist), ext)
+                    src = Gio.File.new_for_path(src_path)
+                    if not src.query_exists():
+                        continue
+                    encoded = self.encode_artist_name(artist)
+                    dst_path = "%s/%s.%s" % (ARTISTS_PATH, encoded, ext)
+                    dst = Gio.File.new_for_path(dst_path)
+                    src.copy(dst, Gio.FileCopyFlags.OVERWRITE, None, None)
+            old_dst = Gio.File.new_for_path(
+                GLib.get_user_data_dir() + "/lollypop/info-backup")
+            old_src.move(old_dst, Gio.FileCopyFlags.OVERWRITE, None, None)
+        except Exception as e:
+            Logger.error(
+                "ArtistArt::__migrate_old_dir(): %s -> %s", e, old_path)
