@@ -11,10 +11,14 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
+from hashlib import md5
+
 from lollypop.define import App, StorageType, ScanUpdate
 from lollypop.objects_track import Track
 from lollypop.objects import Base
 from lollypop.utils import emit_signal
+from lollypop.collection_item import CollectionItem
+from lollypop.logger import Logger
 
 
 class Disc:
@@ -22,12 +26,13 @@ class Disc:
         Represent an album disc
     """
 
-    def __init__(self, album, disc_number, allow_track_skipping):
+    def __init__(self, album, disc_number, storage_type, skipped):
         self.db = App().albums
         self.__tracks = []
         self.__album = album
+        self.__storage_type = storage_type
         self.__number = disc_number
-        self.__allow_track_skipping = allow_track_skipping
+        self.__skipped = skipped
 
     def __del__(self):
         """
@@ -35,6 +40,15 @@ class Disc:
         """
         self.__album = None
 
+    # Used by pickle
+    def __getstate__(self):
+        self.db = None
+        return self.__dict__
+
+    def __setstate__(self, d):
+        self.__dict__.update(d)
+        self.db = App().albums
+
     def set_tracks(self, tracks):
         """
             Set disc tracks
@@ -86,7 +100,8 @@ class Disc:
                 self.album.genre_ids,
                 self.album.artist_ids,
                 self.number,
-                self.__allow_track_skipping)]
+                self.__storage_type,
+                self.__skipped)]
         return self.__tracks
 
 
@@ -100,31 +115,55 @@ class Album(Base):
                 "year": None,
                 "timestamp": 0,
                 "uri": "",
-                "duration": 0,
                 "popularity": 0,
+                "rate": 0,
                 "mtime": 1,
                 "synced": 0,
                 "loved": False,
                 "storage_type": 0,
-                "mb_album_id": None}
+                "mb_album_id": None,
+                "lp_album_id": None}
 
-    def __init__(self, album_id=None, genre_ids=[], artist_ids=[]):
+    def __init__(self, album_id=None, genre_ids=[], artist_ids=[],
+                 skipped=True):
         """
             Init album
             @param album_id as int
             @param genre_ids as [int]
+            @param artist_ids as [int]
+            @param skipped as bool
         """
         Base.__init__(self, App().albums)
         self.id = album_id
         self.genre_ids = genre_ids
         self._tracks = []
         self._discs = []
-        self.__allow_track_skipping = False
+        self.__skipped = skipped
         self.__one_disc = None
+        self.__tracks_storage_type = self.storage_type
         # Use artist ids from db else
         if artist_ids:
+            artists = []
+            for artist_id in set(artist_ids) | set(self.artist_ids):
+                artists.append(App().artists.get_name(artist_id))
+            self.artists = artists
             self.artist_ids = artist_ids
 
+    def __del__(self):
+        """
+            Remove ref cycles
+        """
+        self.reset_tracks()
+
+    # Used by pickle
+    def __getstate__(self):
+        self.db = None
+        return self.__dict__
+
+    def __setstate__(self, d):
+        self.__dict__.update(d)
+        self.db = App().albums
+
     def set_discs(self, discs):
         """
             Set album discs
@@ -134,7 +173,8 @@ class Album(Base):
 
     def set_tracks(self, tracks, clone=True):
         """
-            Set album tracks
+            Set album tracks, do not disable clone if you know self is already
+            used
             @param tracks as [Track]
             @param clone as bool
         """
@@ -144,11 +184,20 @@ class Album(Base):
                 new_track = Track(track.id, self)
                 self._tracks.append(new_track)
         else:
+            # Create a new album for current tracks
+            new_album = Album(self.id, self.genre_ids, self.artist_ids)
+            new_tracks = []
+            for track in self._tracks:
+                if track not in tracks:
+                    track.set_album(new_album)
+                    new_tracks.append(track)
+            new_album._tracks = new_tracks
             self._tracks = tracks
 
     def append_track(self, track, clone=True):
         """
-            Append track to album
+            Append track to album, do not disable clone if you know self is
+            already used
             @param track as Track
             @param clone as bool
         """
@@ -160,7 +209,8 @@ class Album(Base):
 
     def append_tracks(self, tracks, clone=True):
         """
-            Append tracks to album
+            Append tracks to album, do not disable clone if you know self is
+            already used
             @param tracks as [Track]
             @param clone as bool
         """
@@ -184,6 +234,7 @@ class Album(Base):
         """
         self._tracks = []
         self._discs = []
+        self.reset("lp_album_id")
 
     def disc_names(self, disc):
         """
@@ -224,28 +275,49 @@ class Album(Base):
 
     def save(self, save):
         """
-            Save album to collection
+            Save album to collection.
             @param save as bool
         """
+        # Save tracks
+        for track_id in self.track_ids:
+            if save:
+                App().tracks.set_storage_type(track_id, StorageType.SAVED)
+            else:
+                App().tracks.set_storage_type(track_id, StorageType.EPHEMERAL)
+        # Save album
+        self.__save(save)
+
+    def save_track(self, save, track):
+        """
+            Save track to collection
+            @param save as bool
+            @param track as Track
+        """
         if save:
-            self.db.set_storage_type(self.id, StorageType.SAVED)
+            App().tracks.set_storage_type(track.id, StorageType.SAVED)
         else:
-            self.db.set_storage_type(self.id, StorageType.EPHEMERAL)
-        for track in self.tracks:
-            track.save(save)
-        self.reset("mtime")
-        if save:
-            for artist_id in self.artist_ids:
-                emit_signal(App().scanner, "artist-updated",
-                            artist_id, ScanUpdate.ADDED)
-            emit_signal(App().scanner, "album-updated", self.id,
-                        ScanUpdate.ADDED)
-        else:
-            for artist_id in self.artist_ids:
-                emit_signal(App().scanner, "artist-updated", artist_id,
-                            ScanUpdate.REMOVED)
-            emit_signal(App().scanner, "album-updated", self.id,
-                        ScanUpdate.REMOVED)
+            App().tracks.set_storage_type(track.id, StorageType.EPHEMERAL)
+        # Save album
+        self.__save(save)
+
+    def load_tracks(self, cancellable):
+        """
+            Load album tracks from Spotify,
+            do not call this for Storage.COLLECTION
+            @param cancellable as Gio.Cancellable
+            @return status as bool
+        """
+        try:
+            if self.storage_type & (StorageType.COLLECTION |
+                                    StorageType.EXTERNAL):
+                return False
+            elif self.synced != 0 and self.synced != len(self.tracks):
+                from lollypop.search import Search
+                Search().load_tracks(self, cancellable)
+                self.reset_tracks()
+        except Exception as e:
+            Logger.warning("Album::load_tracks(): %s" % e)
+        return True
 
     def set_synced(self, mask):
         """
@@ -255,25 +327,38 @@ class Album(Base):
         self.db.set_synced(self.id, mask)
         self.synced = mask
 
-    def set_skipping_allowed(self):
+    def clone(self, skipped):
         """
-            Mark album as allowing skiiping
-            Calling reset_tracks() needed if already populated
-        """
-        self.__allow_track_skipping = True
-
-    def get_with_skipping_allowed(self):
-        """
-            Get an album clone with skipping activated
+            Clone album
+            @param skipped as bool
             @return album
         """
-        new_album = Album(self.id)
-        tracks = []
-        for track in self.tracks:
-            if track.loved != -1:
-                tracks.append(track)
-        new_album.set_tracks(tracks)
-        return new_album
+        return Album(self.id, self.genre_ids, self.artist_ids, skipped)
+
+    def set_storage_type(self, storage_type):
+        """
+            Set storage type
+            @param storage_type as StorageType
+        """
+        self.__tracks_storage_type = storage_type
+
+    def set_skipped(self):
+        """
+            Set album as skipped, not allowing skipped tracks
+        """
+        self.__skipped = True
+
+    @property
+    def collection_item(self):
+        """
+            Get collection item related to album
+            @return CollectionItem
+        """
+        item = CollectionItem(album_id=self.id,
+                              album_name=self.name,
+                              artist_ids=self.artist_ids,
+                              lp_album_id=self.lp_album_id)
+        return item
 
     @property
     def is_web(self):
@@ -333,7 +418,8 @@ class Album(Base):
         """
         if self.__one_disc is None:
             tracks = self.tracks
-            self.__one_disc = Disc(self, 0, self.__allow_track_skipping)
+            self.__one_disc = Disc(self, 0, self.__tracks_storage_type,
+                                   self.__skipped)
             self.__one_disc.set_tracks(tracks)
         return self.__one_disc
 
@@ -346,7 +432,66 @@ class Album(Base):
         if not self._discs:
             disc_numbers = self.db.get_discs(self.id)
             for disc_number in disc_numbers:
-                disc = Disc(self, disc_number, self.__allow_track_skipping)
+                disc = Disc(self, disc_number,
+                            self.__tracks_storage_type,
+                            self.__skipped)
                 if disc.tracks:
                     self._discs.append(disc)
         return self._discs
+
+    @property
+    def duration(self):
+        """
+            Get album duration and handle caching
+            @return int
+        """
+        if self._tracks:
+            track_ids = [track.lp_track_id for track in self.tracks]
+            track_str = "%s" % sorted(track_ids)
+            track_hash = md5(track_str.encode("utf-8")).hexdigest()
+            album_hash = "%s-%s" % (self.lp_album_id, track_hash)
+        else:
+            album_hash = "%s-%s-%s" % (self.lp_album_id,
+                                       self.genre_ids,
+                                       self.artist_ids)
+        duration = App().cache.get_duration(album_hash)
+        if duration is None:
+            if self._tracks:
+                duration = 0
+                for track in self._tracks:
+                    duration += track.duration
+            else:
+                duration = self.db.get_duration(self.id,
+                                                self.genre_ids,
+                                                self.artist_ids)
+            App().cache.set_duration(self.id, album_hash, duration)
+        return duration
+
+#######################
+# PRIVATE             #
+#######################
+    def __save(self, save):
+        """
+            Save album to collection.
+            @param save as bool
+        """
+        # Save album by updating storage type
+        if save:
+            self.db.set_storage_type(self.id, StorageType.SAVED)
+        else:
+            self.db.set_storage_type(self.id, StorageType.EPHEMERAL)
+        self.reset("mtime")
+        if save:
+            item = CollectionItem(artist_ids=self.artist_ids,
+                                  album_id=self.id)
+            emit_signal(App().scanner, "updated", item,
+                        ScanUpdate.ADDED)
+        else:
+            removed_artist_ids = []
+            for artist_id in self.artist_ids:
+                if not App().artists.get_name(artist_id):
+                    removed_artist_ids.append(artist_id)
+            item = CollectionItem(artist_ids=removed_artist_ids,
+                                  album_id=self.id)
+            emit_signal(App().scanner, "updated", item,
+                        ScanUpdate.REMOVED)
