@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2019 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -24,6 +24,7 @@ class QueuePlayer:
             Init queue
         """
         self.__queue = []
+        self.__queue_current_track = None
 
     def set_queue(self, queue):
         """
@@ -84,14 +85,14 @@ class QueuePlayer:
         if notify:
             emit_signal(self, "queue-changed")
 
-    def track_in_queue(self, track):
+    def is_in_queue(self, track_id):
         """
-            True if track exists in queue
-            @param track as Track
+            True if track id exists in queue
+            @param track_id as int
             @return bool
         """
         if self.__queue:
-            return track.id in self.__queue
+            return track_id in self.__queue
         else:
             return False
 
@@ -123,6 +124,11 @@ class QueuePlayer:
         track_id = None
         if self.__queue:
             track_id = self.__queue[0]
+            if self.__queue_current_track is None:
+                self.__queue_current_track = self._current_track
+        elif self.__queue_current_track is not None:
+            self._current_track = self.__queue_current_track
+            self.__queue_current_track = None
         return Track(track_id)
 
     @property
