@@ -15,12 +15,14 @@ from gi.repository import Gtk, GLib, Gio, Pango
 from gettext import gettext as _
 
 from lollypop.define import App, ArtSize, ViewType, MARGIN, MARGIN_SMALL, Type
+from lollypop.define import ARTISTS_PATH
 from lollypop.objects_album import Album
 from lollypop.helper_art import ArtBehaviour
 from lollypop.information_store import InformationStore
 from lollypop.view_albums_list import AlbumsListView
 from lollypop.view import View
 from lollypop.utils import set_cursor_type, get_network_available
+from lollypop.utils import get_default_storage_type
 from lollypop.helper_gestures import GesturesHelper
 
 
@@ -66,15 +68,14 @@ class InformationView(View):
         View with artist information
     """
 
-    def __init__(self, minimal=False):
+    def __init__(self, view_type, minimal=False):
         """
             Init artist infos
-            @param follow_player as bool
+            @param view_type as ViewType
+            @param minimal as bool
         """
-        View.__init__(self)
+        View.__init__(self, get_default_storage_type(), view_type)
         self.__information_store = InformationStore()
-        self.__information_store.connect("artist-info-changed",
-                                         self.__on_artist_info_changed)
         self.__cancellable = Gio.Cancellable()
         self.__minimal = minimal
         self.__artist_name = ""
@@ -142,17 +143,30 @@ class InformationView(View):
             albums_view.set_size_request(300, -1)
             albums_view.show()
             albums_view.set_margin_start(5)
+            albums_view.add_widget(albums_view.box)
             widget.attach(albums_view, 2, 1, 1, 2)
             albums = []
-            for album_id in App().albums.get_ids([artist_id], []):
+            storage_type = get_default_storage_type()
+            for album_id in App().albums.get_ids([], [artist_id],
+                                                 storage_type, True):
                 albums.append(Album(album_id))
             if not albums:
                 albums = [App().player.current_track.album]
-            # Allows view to be shown without lag
-            GLib.idle_add(albums_view.populate, albums)
-        App().task_helper.run(self.__information_store.get_information,
-                              self.__artist_name,
-                              callback=(self.__set_information_content, True))
+            albums_view.populate(albums)
+        content = self.__information_store.get_information(self.__artist_name,
+                                                           ARTISTS_PATH)
+        if content is None:
+            self.__bio_label.set_text(_("Loading information"))
+            from lollypop.information_downloader import InformationDownloader
+            downloader = InformationDownloader()
+            downloader.get_information(self.__artist_name,
+                                       self.__on_artist_information,
+                                       self.__artist_name)
+        else:
+            App().task_helper.run(
+                GLib.markup_escape_text,
+                content.decode("utf-8"),
+                callback=(self.__bio_label.set_markup,))
 
 #######################
 # PROTECTED           #
@@ -218,35 +232,6 @@ class InformationView(View):
 #######################
 # PRIVATE             #
 #######################
-    def __set_information_content(self, content, initial):
-        """
-            Set information
-            @param content as bytes
-            @param initial as bool => initial loading
-        """
-        if content:
-            self.__bio_label.set_markup(
-                GLib.markup_escape_text(content.decode("utf-8")))
-        elif initial:
-            self.__bio_label.set_text(_("Loading information"))
-            self.__information_store.cache_artist_info(self.__artist_name)
-        else:
-            self.__bio_label.set_text(
-                _("No information for %s") % self.__artist_name)
-
-    def __get_artist_artwork_path_from_cache(self, artist, size):
-        """
-            Get artist artwork path
-            @param artist as str
-            @param size as int
-            @return str
-        """
-        path = InformationStore.get_artwork_path(
-            artist, size, self.get_scale_factor())
-        if path is not None:
-            return path
-        return None
-
     def __on_wikipedia_search_list(self, items):
         """
             Populate view with items
@@ -264,8 +249,8 @@ class InformationView(View):
         """
         if content is not None:
             self.__bio_label.set_text(content.decode("utf-8"))
-            self.__information_store.save_artist_information(
-                self.__artist_name, content)
+            self.__information_store.save_information(
+                self.__artist_name, ARTISTS_PATH, content)
 
     def __on_artist_artwork(self, surface):
         """
@@ -278,14 +263,20 @@ class InformationView(View):
             self.__artist_artwork.set_from_surface(surface)
             del surface
 
-    def __on_artist_info_changed(self, information_store, artist):
+    def __on_artist_information(self, content, artist_name):
         """
-            Update information
-            @param information_store as InformationStore
-            @param artist as str
+            Set label
+            @param content as bytes
+            @param artist_name as str
         """
-        if artist == self.__artist_name:
-            App().task_helper.run(self.__information_store.get_information,
-                                  self.__artist_name,
-                                  callback=(self.__set_information_content,
-                                            False))
+        if artist_name != self.__artist_name:
+            return
+        if content is None:
+            self.__bio_label.set_text(
+                _("No information for %s") % self.__artist_name)
+        else:
+            self.__bio_label.set_markup(
+                GLib.markup_escape_text(content.decode("utf-8")))
+            self.__information_store.save_information(self.__artist_name,
+                                                      ARTISTS_PATH,
+                                                      content)
